@@ -1,3 +1,26 @@
+# Copyright (c) 2014  Timm Murray
+# All rights reserved.
+# 
+# Redistribution and use in source and binary forms, with or without 
+# modification, are permitted provided that the following conditions are met:
+# 
+#     * Redistributions of source code must retain the above copyright notice, 
+#       this list of conditions and the following disclaimer.
+#     * Redistributions in binary form must reproduce the above copyright 
+#       notice, this list of conditions and the following disclaimer in the 
+#       documentation and/or other materials provided with the distribution.
+# 
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
+# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
+# POSSIBILITY OF SUCH DAMAGE.
 package UAV::Pilot::Commands;
 use v5.14;
 use Moose;
@@ -6,10 +29,7 @@ use File::Spec;
 
 use constant MOD_EXTENSION => '.uav';
 
-has 'device' => (
-    is   => 'ro',
-    does => 'UAV::Pilot::Device',
-);
+
 has 'lib_dirs' => (
     is      => 'ro',
     isa     => 'ArrayRef[Str]',
@@ -19,8 +39,29 @@ has 'lib_dirs' => (
         add_lib_dir => 'push',
     },
 );
+has 'condvar' => (
+    is  => 'ro',
+    isa => 'AnyEvent::CondVar',
+);
+has 'controller_callback_ardrone' => (
+    is  => 'ro',
+    isa => 'CodeRef',
+);
+has 'controller_callback_wumpusrover' => (
+    is  => 'ro',
+    isa => 'CodeRef',
+);
+has 'quit_subs' => (
+    traits  => ['Array'],
+    is      => 'ro',
+    isa     => 'ArrayRef[CodeRef]',
+    default => sub {[]},
+    handles => {
+        '_push_quit_sub' => 'push',
+    },
+);
 
-our ($dev, $s);
+our $s;
 
 #
 # Sole command that can run without loading other libraries
@@ -28,6 +69,7 @@ our ($dev, $s);
 sub load ($;$)
 {
     my ($mod_name, $args) = @_;
+    $$args{condvar} = $s->condvar unless exists $$args{condvar};
     $s->load_lib( $mod_name, $args );
 }
 
@@ -41,20 +83,27 @@ sub run_cmd
     }
     return 1 unless defined $cmd;
 
-    $s   = $self;
-    $dev = $self->device;
+    $s = $self;
     eval $cmd;
     die $@ if $@;
 
     return 1;
 }
 
+sub quit
+{
+    my ($self) = @_;
+    $_->() for @{ $self->{quit_subs} };
+    return 1;
+}
+
 
 sub load_lib
 {
     my ($self, $mod_name, $args) = @_;
     my @search_dirs = @{ $self->lib_dirs };
     my $mod_file = $mod_name . $self->MOD_EXTENSION;
+    my $host = delete $$args{host};
 
     my $found = 0;
     foreach my $dir (@search_dirs) {
@@ -93,7 +142,7 @@ sub _compile_mod
 
     $pack = ref($self) unless defined $pack;
     if( my $call = $pack->can( 'uav_module_init' ) ) {
-        $call->( $pack, $args );
+        $call->( $pack, $self, $args );
 
         # Clear uav_module_init.  Would prefer a solution without eval( STRING ), 
         # though a symbol table manipulation method may be considered just as evil.
@@ -101,6 +150,10 @@ sub _compile_mod
         eval $del_str;
     }
 
+    if( my $quit_call = $pack->can( 'uav_module_quit' ) ) {
+        $self->_push_quit_sub( $quit_call );
+    }
+
     return 1;
 }
 
@@ -117,9 +170,11 @@ __END__
 
 =head1 SYNOPSIS
 
-    my $device; # Some UAV::Pilot::Device instance, defined elsewhere
+    my $device; # Some UAV::Pilot::Control instance, defined elsewhere
     my $cmds = UAV::Pilot::Commands->new({
         device => $device,
+        controller_callback_ardrone     => \&make_ardrone_controller,
+        controller_callback_wumpusrover => \&make_wumpusrover_controller,
     });
     
     $cmds->load_lib( 'ARDrone' );
@@ -136,10 +191,21 @@ REPL shells.
 =head2 new
 
     new({
-        device => $device
+        condvar                         => $cv,
+        controller_callback_ardrone     => sub { ... },
+        controller_callback_wumpusrover => sub { .. },
     })
 
-Constructor.  Takes a L<UAV::Pilot::Device> instance.
+Constructor.  The C<condvar> parameter is an C<AnyEvent::Condvar>.
+
+The C<controller_callback_*> parameters take a sub ref.  The subroutines take 
+a the parameters C<($cmd, $cv, $easy_event)>, where C<$cmd> is this 
+C<UAV::Pilot::Commands> instance, C<$cv> is the condvar passed above, and 
+C<$easy_event> is an C<UAV::Pilot::EasyEvent> instance.  It should return a 
+C<UAV::Pilot::Control> object of the associated type (generally one of the 
+C<*::Event> types with C<init_event_loop()> called).
+
+Note that this API is likely to change to a factory pattern in the near future.
 
 =head2 load_lib
 
@@ -170,9 +236,9 @@ Executes a command.  Note that this will execute arbitrary Perl statements.
 =head1 COMMANDS
 
 Commands provide an easy interface for writing simple UAV programms in a REPL shell.  
-They are usually thin interfaces over a L<UAV::Pilot::Device>.  If you're writing a 
+They are usually thin interfaces over a L<UAV::Pilot::Control>.  If you're writing a 
 complicated script, it's suggested that you skip this interface and write to the 
-L<UAV::Pilot::Device> directly.
+L<UAV::Pilot::Control> directly.
 
 =head2 load
 
@@ -196,7 +262,12 @@ subroutine.
 
 =head1 WRITING YOUR OWN EXTENSIONS
 
-Extensions should go under the directory C<UAV/Pilot/Modules/> with a C<.uav> extension. 
+Extensions should go in the directory specified by:
+
+    File::ShareDir::dist_dir( 'UAV-Pilot' )
+
+They should have a C<.uav> extension.
+
 You write them much like any Perl module, but don't use a C<package> statement--the package
 will be controlled by C<UAV::Pilot::Command> when loaded.  Like a Perl module, it should 
 return true as its final statement (put a C<1;> at the end).
@@ -210,3 +281,5 @@ parens.
 The method C<uav_module_init()> is called with the package name as the first argument.  
 Subsquent arguments will be the hashref passed to C<load()/load_lib()>.  After being called,
 this sub will be deleted from the package.
+
+The method C<uav_module_quit()> is called when the REPL is closing.
